@@ -797,11 +797,11 @@ struct find_best_w_mers
 	typedef typename storage_policy::container                            container_t;       ///< Container type to store best W-mers.
 	typedef boost::shared_ptr< container_t >                              container_ptr;     ///< Pointer to best W-mers.
 
-	size_t                              num_to_find;      ///< Number to find.
-	container_ptr                       best_w_mers;      ///< Best w_mers.
-	storage_policy                      storage;          ///< Stores the best W-mers.
+	size_t                               num_to_find;      ///< Number to find.
+	container_ptr                        best_w_mers;      ///< Best w_mers.
+	storage_policy                       storage;          ///< Stores the best W-mers.
 	double                              worst_Z;          ///< The worst Z so far.
-	double                              lZtt;             ///< The log Z threshold term.
+	double                              lZt;             ///< The log Z threshold term.
 
 
 	/** Constructor. */
@@ -824,18 +824,18 @@ struct find_best_w_mers
 	void
 	on_descent_begin() {
 		base_t::stats.reset();
-		worst_Z = lZtt = -std::numeric_limits< double >::max();
+		worst_Z = lZt = -std::numeric_limits< double >::max();
 		storage.initialise();
 	}
 
 
 	/**
-	 * \return The threshold on log(<Z>/(1-<Z>)).
+	 * \return The threshold on log(<Z>).
 	 */
 	double
-	get_log_Z_threshold_term() {
+	get_log_Z_threshold() {
 		if( storage.stored_enough() ) {
-			return lZtt;
+			return lZt;
 		} else {
 			return -std::numeric_limits< double >::max();
 		}
@@ -850,7 +850,7 @@ struct find_best_w_mers
 	update_worst_Z( double new_worst_Z ) {
 		if( new_worst_Z != worst_Z ) {
 			worst_Z = new_worst_Z;
-			lZtt = std::log( new_worst_Z / ( 1. - new_worst_Z ) );
+			lZt = std::log( new_worst_Z );
 		}
 	}
 
