@@ -6,6 +6,7 @@
 #include "steme_python_defs.h"
 
 #include <steme/background_model.h>
+#include <steme/background_model_on_the_fly.h>
 #include <steme/data.h>
 
 #include <myrrh/python/boost_range.h>
@@ -16,7 +17,13 @@ namespace py = boost::python;
 using namespace steme;
 using namespace steme::python;
 
-typedef data< EXPOSED_STEME_SPEC > data_t;
+typedef data< STEME_INDEX_MODULE_SPEC > data_t;
+typedef typename background_model_meta< STEME_INDEX_MODULE_SPEC >::type bg_model_t;
+
+
+namespace steme {
+const double log_quarter = std::log( 0.25 ); /**< log(.25) */
+} //namespace steme
 
 
 
@@ -46,6 +53,9 @@ zero_order_from_seq( py::object py_seq )
 
 
 
+#if STEME_INDEX_MODULE_TYPE == STEME_INDEX_DEFAULT_TYPE
+
+
 bg_model::ptr
 create_bg_model_from_0_order(
     size_t W,
@@ -91,19 +101,79 @@ create_bg_model_from_Markov_model(
 }
 
 
-/// Registers a create background from Markov model of given order
-#define REGISTER_CREATE_BG_FROM_MARKOV_MODEL_OF_ORDER( z, Order, text ) \
-    py::def( \
-        MYRRH_MAKE_STRING( "create_bg_model_from_Markov_model_" << Order ).c_str(), \
-        create_bg_model_from_Markov_model< Order >, \
-        ( \
-            py::arg( "W" ), \
-            py::arg( "data" ), \
-            py::arg( "model" ), \
-            py::arg( "freqs" ) \
-        ), \
-        "Creates a background model from the Markov model." \
-    );
+
+#else //STEME_INDEX_MODULE_TYPE == STEME_INDEX_DEFAULT_TYPE
+
+template< typename MultiArray >
+double
+get_smallest_ll( const MultiArray & x ) {
+    if( x.empty() ) {
+        throw std::logic_error( "Markov model has no likelihoods." );
+    }
+    return *std::min_element( x.data(), x.data() + x.num_elements() );
+}
+
+
+
+/// Calculates the likelihood of a infix in context on the fly.
+template< typename MarkovModel >
+struct markov_on_the_fly_evaluator
+: on_the_fly_evaluator< STEME_INDEX_MODULE_SPEC >
+{
+    const MarkovModel & mm; ///< The Markov model
+    double smallest_ll; ///< The smallest log likelihood in the Markov model.
+
+    markov_on_the_fly_evaluator( const MarkovModel & mm )
+    : mm( mm )
+    , smallest_ll( get_smallest_ll( mm.mm.storage ) )
+    { }
+
+    virtual
+    ~markov_on_the_fly_evaluator()
+    { }
+
+    struct addition
+    {
+        addition( double & x ) : x( x ) { }
+        void operator()( double y ) const { x += y; }
+        double & x;
+    };
+
+    virtual
+    double
+    evaluate( it context, it begin, it end ) {
+        double result = 0.;
+        mm.evaluate_in_context(
+            context,
+            begin,
+            end,
+            boost::make_function_output_iterator( addition( result ) )
+        );
+        return result;
+
+    }
+
+    virtual
+    double
+    min_ll( size_t w, size_t W ) {
+        return ( W - w ) * smallest_ll;
+    }
+};
+
+
+
+
+template< size_t Order >
+on_the_fly_bg_model< STEME_INDEX_MODULE_SPEC >::ptr
+create_on_the_fly_bg_model_from_Markov_model(
+    const complete_markov_model< Order, 4, double > & model
+) {
+    typename on_the_fly_evaluator< STEME_INDEX_MODULE_SPEC >::ptr evaluator( new markov_on_the_fly_evaluator< complete_markov_model< Order, 4, double > >( model ) );
+    return on_the_fly_bg_model< STEME_INDEX_MODULE_SPEC >::ptr( new on_the_fly_bg_model< STEME_INDEX_MODULE_SPEC >( evaluator ) );
+}
+
+
+#endif //STEME_INDEX_MODULE_TYPE == STEME_INDEX_DEFAULT_TYPE
 
 
 
@@ -135,21 +205,21 @@ expose_bg() {
 
 
 
+
     //
     // Background model
     //
     py::class_<
-        bg_model,
+        bg_model_t,
         boost::noncopyable,
-        bg_model::ptr
+        bg_model_t::ptr
     > bg_model_class(
         "BgModel",
-        "Likelihoods of W-mers and 0-order frequencies.",
+        "Models the likelihoods of W-mers under a background model.",
         py::no_init
     );
-    bg_model_class.def_readonly( "freqs", &bg_model::freqs, "The background frequencies." );
-
-
+#if STEME_INDEX_MODULE_TYPE == STEME_INDEX_DEFAULT_TYPE
+    bg_model_class.def_readonly( "freqs", &bg_model_t::freqs, "The background frequencies." );
 
 
     py::def(
@@ -174,11 +244,54 @@ expose_bg() {
             py::arg( "base_LLs" ),
             py::arg( "freqs" )
         ),
-        "Creates a background model from likelihoods given per-base."
-        // following line does not work has runtime error: TypeError: cannot create weak reference to 'int' object
-        // py::with_custodian_and_ward< 1, 4 >() // don't let base_LLs be destroyed before the background model
+        "Creates a background model from likelihoods given per-base.",
+        py::with_custodian_and_ward_postcall<
+            0,
+            2,
+            py::with_custodian_and_ward_postcall< 0, 3 >
+        >() // don't let base_LLs or data be destroyed before the background model
+    );
+
+
+/// Registers a create background from Markov model of given order
+# define REGISTER_CREATE_BG_FROM_MARKOV_MODEL_OF_ORDER( z, Order, text ) \
+    py::def( \
+        MYRRH_MAKE_STRING( "create_bg_model_from_Markov_model_" << Order ).c_str(), \
+        create_bg_model_from_Markov_model< Order >, \
+        ( \
+            py::arg( "W" ), \
+            py::arg( "data" ), \
+            py::arg( "model" ), \
+            py::arg( "freqs" ) \
+        ), \
+        "Creates a background model from the Markov model." \
     );
 
     // define create from Markov model functions.
     BOOST_PP_REPEAT_FROM_TO( 0, STEME_MAX_MARKOV_MODEL_ORDER, REGISTER_CREATE_BG_FROM_MARKOV_MODEL_OF_ORDER, )
+
+
+
+
+#else //STEME_INDEX_MODULE_TYPE == STEME_INDEX_DEFAULT_TYPE
+
+
+/// Registers a create background from Markov model of given order
+# define REGISTER_CREATE_BG_FROM_MARKOV_MODEL_OF_ORDER( z, Order, text ) \
+    py::def( \
+        MYRRH_MAKE_STRING( "create_bg_model_from_Markov_model_" << Order ).c_str(), \
+        create_on_the_fly_bg_model_from_Markov_model< Order >, \
+        ( \
+            py::arg( "model" ) \
+        ), \
+        "Creates a background model from the Markov model." \
+    );
+
+
+    // define create from Markov model functions.
+    BOOST_PP_REPEAT_FROM_TO( 0, STEME_MAX_MARKOV_MODEL_ORDER, REGISTER_CREATE_BG_FROM_MARKOV_MODEL_OF_ORDER, )
+
+
+#endif //STEME_INDEX_MODULE_TYPE == STEME_INDEX_DEFAULT_TYPE
+
 }
