@@ -12,7 +12,13 @@ namespace py = boost::python;
 using namespace steme;
 using namespace steme::python;
 
-typedef data< EXPOSED_STEME_SPEC > data_t;
+typedef data< STEME_INDEX_MODULE_SPEC > data_t;
+
+
+
+
+
+
 
 
 
