@@ -177,20 +177,109 @@ BOOST_AUTO_TEST_CASE( markov_model_evaluation_test )
 		BOOST_CHECK_CLOSE( probs[5] - probs[4], std::log( 1./4. ), 0.0001 );
 	}
 
-	{
-		vector< size_t > eval_seq = list_of(0)(1)(2)(3)(0)(1)(2)(3);
-		vector< double > probs;
-		model.evaluate( eval_seq.begin(), eval_seq.end(), boost::make_function_output_iterator( make_push_back_cumulative( probs ) ) );
-		BOOST_CHECK_EQUAL( eval_seq.size(), probs.size() );
-		BOOST_CHECK_CLOSE( probs[0], std::log( 3./9. ), 0.0001 );
-		BOOST_CHECK_CLOSE( probs[1] - probs[0], std::log( 2./5. ), 0.0001 );
-		BOOST_CHECK_CLOSE( probs[2] - probs[1], std::log( 2./5. ), 0.0001 );
-		BOOST_CHECK_CLOSE( probs[3] - probs[2], std::log( 2./5. ), 0.0001 );
-		BOOST_CHECK_CLOSE( probs[4] - probs[3], std::log( 2./5. ), 0.0001 );
-		BOOST_CHECK_CLOSE( probs[5] - probs[4], std::log( 1./4. ), 0.0001 );
-		BOOST_CHECK_CLOSE( probs[6] - probs[5], std::log( 1./4. ), 0.0001 );
-		BOOST_CHECK_CLOSE( probs[7] - probs[6], std::log( 2./5. ), 0.0001 );
-	}
+    {
+        vector< size_t > eval_seq = list_of(0)(1)(2)(3)(0)(1)(2)(3);
+        vector< double > probs;
+        model.evaluate( eval_seq.begin(), eval_seq.end(), boost::make_function_output_iterator( make_push_back_cumulative( probs ) ) );
+        BOOST_CHECK_EQUAL( eval_seq.size(), probs.size() );
+        BOOST_CHECK_CLOSE( probs[0], std::log( 3./9. ), 0.0001 );
+        BOOST_CHECK_CLOSE( probs[1] - probs[0], std::log( 2./5. ), 0.0001 );
+        BOOST_CHECK_CLOSE( probs[2] - probs[1], std::log( 2./5. ), 0.0001 );
+        BOOST_CHECK_CLOSE( probs[3] - probs[2], std::log( 2./5. ), 0.0001 );
+        BOOST_CHECK_CLOSE( probs[4] - probs[3], std::log( 2./5. ), 0.0001 );
+        BOOST_CHECK_CLOSE( probs[5] - probs[4], std::log( 1./4. ), 0.0001 );
+        BOOST_CHECK_CLOSE( probs[6] - probs[5], std::log( 1./4. ), 0.0001 );
+        BOOST_CHECK_CLOSE( probs[7] - probs[6], std::log( 2./5. ), 0.0001 );
+    }
+
+    // test evaluation within context, when context is not large enough
+    {
+        vector< size_t > eval_seq = list_of(0)(1)(2)(3)(0)(1)(2)(3);
+        vector< double > probs;
+        model.evaluate_in_context(
+            eval_seq.begin(),
+            eval_seq.begin() + 1,
+            eval_seq.end(),
+            boost::make_function_output_iterator( make_push_back_cumulative( probs ) )
+        );
+        BOOST_CHECK_EQUAL( eval_seq.size() - 1, probs.size() );
+        BOOST_CHECK_CLOSE( probs[0], std::log( 2./5. ), 0.0001 );
+        BOOST_CHECK_CLOSE( probs[1] - probs[0], std::log( 2./5. ), 0.0001 );
+        BOOST_CHECK_CLOSE( probs[2] - probs[1], std::log( 2./5. ), 0.0001 );
+        BOOST_CHECK_CLOSE( probs[3] - probs[2], std::log( 2./5. ), 0.0001 );
+        BOOST_CHECK_CLOSE( probs[4] - probs[3], std::log( 1./4. ), 0.0001 );
+        BOOST_CHECK_CLOSE( probs[5] - probs[4], std::log( 1./4. ), 0.0001 );
+        BOOST_CHECK_CLOSE( probs[6] - probs[5], std::log( 2./5. ), 0.0001 );
+    }
+
+    // test evaluation within context, when context is not large enough
+    {
+        vector< size_t > eval_seq = list_of(0)(1)(2)(3)(0)(1)(2)(3);
+        vector< double > probs;
+        model.evaluate_in_context(
+            eval_seq.begin(),
+            eval_seq.begin() + 2,
+            eval_seq.end(),
+            boost::make_function_output_iterator( make_push_back_cumulative( probs ) )
+        );
+        BOOST_CHECK_EQUAL( eval_seq.size() - 2, probs.size() );
+        BOOST_CHECK_CLOSE( probs[0], std::log( 2./5. ), 0.0001 );
+        BOOST_CHECK_CLOSE( probs[1] - probs[0], std::log( 2./5. ), 0.0001 );
+        BOOST_CHECK_CLOSE( probs[2] - probs[1], std::log( 2./5. ), 0.0001 );
+        BOOST_CHECK_CLOSE( probs[3] - probs[2], std::log( 1./4. ), 0.0001 );
+        BOOST_CHECK_CLOSE( probs[4] - probs[3], std::log( 1./4. ), 0.0001 );
+        BOOST_CHECK_CLOSE( probs[5] - probs[4], std::log( 2./5. ), 0.0001 );
+    }
+
+    // test evaluation within context, when context is just right size
+    {
+        vector< size_t > eval_seq = list_of(0)(1)(2)(3)(0)(1)(2)(3);
+        vector< double > probs;
+        model.evaluate_in_context(
+            eval_seq.begin(),
+            eval_seq.begin() + 3,
+            eval_seq.end(),
+            boost::make_function_output_iterator( make_push_back_cumulative( probs ) )
+        );
+        BOOST_CHECK_EQUAL( eval_seq.size() - 3, probs.size() );
+        BOOST_CHECK_CLOSE( probs[0], std::log( 2./5. ), 0.0001 );
+        BOOST_CHECK_CLOSE( probs[1] - probs[0], std::log( 2./5. ), 0.0001 );
+        BOOST_CHECK_CLOSE( probs[2] - probs[1], std::log( 1./4. ), 0.0001 );
+        BOOST_CHECK_CLOSE( probs[3] - probs[2], std::log( 1./4. ), 0.0001 );
+        BOOST_CHECK_CLOSE( probs[4] - probs[3], std::log( 2./5. ), 0.0001 );
+    }
+
+    // test evaluation within context, when context is large enough
+    {
+        vector< size_t > eval_seq = list_of(0)(1)(2)(3)(0)(1)(2)(3);
+        vector< double > probs;
+        model.evaluate_in_context(
+            eval_seq.begin(),
+            eval_seq.begin() + 4,
+            eval_seq.end(),
+            boost::make_function_output_iterator( make_push_back_cumulative( probs ) )
+        );
+        BOOST_CHECK_EQUAL( eval_seq.size() - 4, probs.size() );
+        BOOST_CHECK_CLOSE( probs[0], std::log( 2./5. ), 0.0001 );
+        BOOST_CHECK_CLOSE( probs[1] - probs[0], std::log( 1./4. ), 0.0001 );
+        BOOST_CHECK_CLOSE( probs[2] - probs[1], std::log( 1./4. ), 0.0001 );
+        BOOST_CHECK_CLOSE( probs[3] - probs[2], std::log( 2./5. ), 0.0001 );
+    }
+
+    // test evaluation within context, when context is large enough
+    {
+        vector< size_t > eval_seq = list_of(0)(1)(2)(3)(0)(1)(2)(3);
+        vector< double > probs;
+        model.evaluate_in_context(
+            eval_seq.begin(),
+            eval_seq.begin() + 6,
+            eval_seq.end(),
+            boost::make_function_output_iterator( make_push_back_cumulative( probs ) )
+        );
+        BOOST_CHECK_EQUAL( eval_seq.size() - 6, probs.size() );
+        BOOST_CHECK_CLOSE( probs[0], std::log( 1./4. ), 0.0001 );
+        BOOST_CHECK_CLOSE( probs[1] - probs[0], std::log( 2./5. ), 0.0001 );
+    }
 }
 
 
